@@ -1,20 +1,14 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import
-
 import inspect
 import sys
 import threading
 import weakref
 from importlib import import_module
 
-from sentry_sdk._compat import string_types, text_type
-from sentry_sdk._types import TYPE_CHECKING
+import sentry_sdk
 from sentry_sdk.consts import OP, SPANDATA
-from sentry_sdk.db.explain_plan.django import attach_explain_plan_to_span
-from sentry_sdk.hub import Hub, _should_send_default_pii
-from sentry_sdk.scope import add_global_event_processor
-from sentry_sdk.serializer import add_global_repr_processor
-from sentry_sdk.tracing import SOURCE_FOR_STYLE, TRANSACTION_SOURCE_URL
+from sentry_sdk.scope import add_global_event_processor, should_send_default_pii
+from sentry_sdk.serializer import add_global_repr_processor, add_repr_sequence_type
+from sentry_sdk.tracing import SOURCE_FOR_STYLE, TransactionSource
 from sentry_sdk.tracing_utils import add_query_source, record_sql_queries
 from sentry_sdk.utils import (
     AnnotatedValue,
@@ -23,14 +17,18 @@ from sentry_sdk.utils import (
     SENSITIVE_DATA_SUBSTITUTE,
     logger,
     capture_internal_exceptions,
+    ensure_integration_enabled,
     event_from_exception,
     transaction_from_function,
     walk_exception_chain,
 )
-from sentry_sdk.integrations import Integration, DidNotEnable
+from sentry_sdk.integrations import _check_minimum_version, Integration, DidNotEnable
 from sentry_sdk.integrations.logging import ignore_logger
 from sentry_sdk.integrations.wsgi import SentryWsgiMiddleware
-from sentry_sdk.integrations._wsgi_common import RequestExtractor
+from sentry_sdk.integrations._wsgi_common import (
+    DEFAULT_HTTP_METHODS_TO_CAPTURE,
+    RequestExtractor,
+)
 
 try:
     from django import VERSION as DJANGO_VERSION
@@ -71,6 +69,7 @@ if DJANGO_VERSION[:2] > (1, 8):
 else:
     patch_caching = None  # type: ignore
 
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
     from typing import Any
@@ -86,7 +85,6 @@ if TYPE_CHECKING:
     from django.utils.datastructures import MultiValueDict
 
     from sentry_sdk.tracing import Span
-    from sentry_sdk.scope import Scope
     from sentry_sdk.integrations.wsgi import _ScopedResponse
     from sentry_sdk._types import Event, Hint, EventProcessor, NotImplementedType
 
@@ -108,7 +106,19 @@ TRANSACTION_STYLE_VALUES = ("function_name", "url")
 
 
 class DjangoIntegration(Integration):
+    """
+    Auto instrument a Django application.
+
+    :param transaction_style: How to derive transaction names. Either `"function_name"` or `"url"`. Defaults to `"url"`.
+    :param middleware_spans: Whether to create spans for middleware. Defaults to `True`.
+    :param signals_spans: Whether to create spans for signals. Defaults to `True`.
+    :param signals_denylist: A list of signals to ignore when creating spans.
+    :param cache_spans: Whether to create spans for cache operations. Defaults to `False`.
+    """
+
     identifier = "django"
+    origin = f"auto.http.{identifier}"
+    origin_db = f"auto.db.{identifier}"
 
     transaction_style = ""
     middleware_spans = None
@@ -118,13 +128,14 @@ class DjangoIntegration(Integration):
 
     def __init__(
         self,
-        transaction_style="url",
-        middleware_spans=True,
-        signals_spans=True,
-        cache_spans=False,
-        signals_denylist=None,
+        transaction_style="url",  # type: str
+        middleware_spans=True,  # type: bool
+        signals_spans=True,  # type: bool
+        cache_spans=False,  # type: bool
+        signals_denylist=None,  # type: Optional[list[signals.Signal]]
+        http_methods_to_capture=DEFAULT_HTTP_METHODS_TO_CAPTURE,  # type: tuple[str, ...]
     ):
-        # type: (str, bool, bool, bool, Optional[list[signals.Signal]]) -> None
+        # type: (...) -> None
         if transaction_style not in TRANSACTION_STYLE_VALUES:
             raise ValueError(
                 "Invalid value for transaction_style: %s (must be in %s)"
@@ -132,16 +143,18 @@ class DjangoIntegration(Integration):
             )
         self.transaction_style = transaction_style
         self.middleware_spans = middleware_spans
+
         self.signals_spans = signals_spans
-        self.cache_spans = cache_spans
         self.signals_denylist = signals_denylist or []
 
+        self.cache_spans = cache_spans
+
+        self.http_methods_to_capture = tuple(map(str.upper, http_methods_to_capture))
+
     @staticmethod
     def setup_once():
         # type: () -> None
-
-        if DJANGO_VERSION < (1, 8):
-            raise DidNotEnable("Django 1.8 or newer is required.")
+        _check_minimum_version(DjangoIntegration, DJANGO_VERSION)
 
         install_sql_hook()
         # Patch in our custom middleware.
@@ -154,20 +167,28 @@ class DjangoIntegration(Integration):
 
         old_app = WSGIHandler.__call__
 
+        @ensure_integration_enabled(DjangoIntegration, old_app)
         def sentry_patched_wsgi_handler(self, environ, start_response):
             # type: (Any, Dict[str, str], Callable[..., Any]) -> _ScopedResponse
-            if Hub.current.get_integration(DjangoIntegration) is None:
-                return old_app(self, environ, start_response)
-
             bound_old_app = old_app.__get__(self, WSGIHandler)
 
             from django.conf import settings
 
             use_x_forwarded_for = settings.USE_X_FORWARDED_HOST
 
-            return SentryWsgiMiddleware(bound_old_app, use_x_forwarded_for)(
-                environ, start_response
+            integration = sentry_sdk.get_client().get_integration(DjangoIntegration)
+
+            middleware = SentryWsgiMiddleware(
+                bound_old_app,
+                use_x_forwarded_for,
+                span_origin=DjangoIntegration.origin,
+                http_methods_to_capture=(
+                    integration.http_methods_to_capture
+                    if integration
+                    else DEFAULT_HTTP_METHODS_TO_CAPTURE
+                ),
             )
+            return middleware(environ, start_response)
 
         WSGIHandler.__call__ = sentry_patched_wsgi_handler
 
@@ -237,11 +258,6 @@ class DjangoIntegration(Integration):
             if not isinstance(value, QuerySet) or value._result_cache:
                 return NotImplemented
 
-            # Do not call Hub.get_integration here. It is intentional that
-            # running under a new hub does not suddenly start executing
-            # querysets. This might be surprising to the user but it's likely
-            # less annoying.
-
             return "<%s from %s at 0x%x>" % (
                 value.__class__.__name__,
                 value.__module__,
@@ -253,6 +269,7 @@ class DjangoIntegration(Integration):
         patch_views()
         patch_templates()
         patch_signals()
+        add_template_context_repr_sequence()
 
         if patch_caching is not None:
             patch_caching()
@@ -365,7 +382,7 @@ def _patch_django_asgi_handler():
 
 
 def _set_transaction_name_and_source(scope, transaction_style, request):
-    # type: (Scope, str, WSGIRequest) -> None
+    # type: (sentry_sdk.Scope, str, WSGIRequest) -> None
     try:
         transaction_name = None
         if transaction_style == "function_name":
@@ -382,7 +399,7 @@ def _set_transaction_name_and_source(scope, transaction_style, request):
 
         if transaction_name is None:
             transaction_name = request.path_info
-            source = TRANSACTION_SOURCE_URL
+            source = TransactionSource.URL
         else:
             source = SOURCE_FOR_STYLE[transaction_style]
 
@@ -396,7 +413,7 @@ def _set_transaction_name_and_source(scope, transaction_style, request):
         # So we don't check here what style is configured
         if hasattr(urlconf, "handler404"):
             handler = urlconf.handler404
-            if isinstance(handler, string_types):
+            if isinstance(handler, str):
                 scope.transaction = handler
             else:
                 scope.transaction = transaction_from_function(
@@ -408,24 +425,23 @@ def _set_transaction_name_and_source(scope, transaction_style, request):
 
 def _before_get_response(request):
     # type: (WSGIRequest) -> None
-    hub = Hub.current
-    integration = hub.get_integration(DjangoIntegration)
+    integration = sentry_sdk.get_client().get_integration(DjangoIntegration)
     if integration is None:
         return
 
     _patch_drf()
 
-    with hub.configure_scope() as scope:
-        # Rely on WSGI middleware to start a trace
-        _set_transaction_name_and_source(scope, integration.transaction_style, request)
+    scope = sentry_sdk.get_current_scope()
+    # Rely on WSGI middleware to start a trace
+    _set_transaction_name_and_source(scope, integration.transaction_style, request)
 
-        scope.add_event_processor(
-            _make_wsgi_request_event_processor(weakref.ref(request), integration)
-        )
+    scope.add_event_processor(
+        _make_wsgi_request_event_processor(weakref.ref(request), integration)
+    )
 
 
 def _attempt_resolve_again(request, scope, transaction_style):
-    # type: (WSGIRequest, Scope, str) -> None
+    # type: (WSGIRequest, sentry_sdk.Scope, str) -> None
     """
     Some django middlewares overwrite request.urlconf
     so we need to respect that contract,
@@ -439,13 +455,12 @@ def _attempt_resolve_again(request, scope, transaction_style):
 
 def _after_get_response(request):
     # type: (WSGIRequest) -> None
-    hub = Hub.current
-    integration = hub.get_integration(DjangoIntegration)
+    integration = sentry_sdk.get_client().get_integration(DjangoIntegration)
     if integration is None or integration.transaction_style != "url":
         return
 
-    with hub.configure_scope() as scope:
-        _attempt_resolve_again(request, scope, integration.transaction_style)
+    scope = sentry_sdk.get_current_scope()
+    _attempt_resolve_again(request, scope, integration.transaction_style)
 
 
 def _patch_get_response():
@@ -488,17 +503,10 @@ def _make_wsgi_request_event_processor(weak_request, integration):
             # We have a `asgi_request_event_processor` for this.
             return event
 
-        try:
-            drf_request = request._sentry_drf_request_backref()
-            if drf_request is not None:
-                request = drf_request
-        except AttributeError:
-            pass
-
         with capture_internal_exceptions():
             DjangoRequestExtractor(request).extract_into_event(event)
 
-        if _should_send_default_pii():
+        if should_send_default_pii():
             with capture_internal_exceptions():
                 _set_user_info(request, event)
 
@@ -509,25 +517,34 @@ def _make_wsgi_request_event_processor(weak_request, integration):
 
 def _got_request_exception(request=None, **kwargs):
     # type: (WSGIRequest, **Any) -> None
-    hub = Hub.current
-    integration = hub.get_integration(DjangoIntegration)
-    if integration is not None:
-        if request is not None and integration.transaction_style == "url":
-            with hub.configure_scope() as scope:
-                _attempt_resolve_again(request, scope, integration.transaction_style)
+    client = sentry_sdk.get_client()
+    integration = client.get_integration(DjangoIntegration)
+    if integration is None:
+        return
 
-        # If an integration is there, a client has to be there.
-        client = hub.client  # type: Any
+    if request is not None and integration.transaction_style == "url":
+        scope = sentry_sdk.get_current_scope()
+        _attempt_resolve_again(request, scope, integration.transaction_style)
 
-        event, hint = event_from_exception(
-            sys.exc_info(),
-            client_options=client.options,
-            mechanism={"type": "django", "handled": False},
-        )
-        hub.capture_event(event, hint=hint)
+    event, hint = event_from_exception(
+        sys.exc_info(),
+        client_options=client.options,
+        mechanism={"type": "django", "handled": False},
+    )
+    sentry_sdk.capture_event(event, hint=hint)
 
 
 class DjangoRequestExtractor(RequestExtractor):
+    def __init__(self, request):
+        # type: (Union[WSGIRequest, ASGIRequest]) -> None
+        try:
+            drf_request = request._sentry_drf_request_backref()
+            if drf_request is not None:
+                request = drf_request
+        except AttributeError:
+            pass
+        self.request = request
+
     def env(self):
         # type: () -> Dict[str, str]
         return self.request.META
@@ -568,7 +585,7 @@ class DjangoRequestExtractor(RequestExtractor):
         # type: () -> Optional[Dict[str, Any]]
         try:
             return self.request.data
-        except AttributeError:
+        except Exception:
             return RequestExtractor.parsed_body(self)
 
 
@@ -620,62 +637,56 @@ def install_sql_hook():
         # This won't work on Django versions < 1.6
         return
 
+    @ensure_integration_enabled(DjangoIntegration, real_execute)
     def execute(self, sql, params=None):
         # type: (CursorWrapper, Any, Optional[Any]) -> Any
-        hub = Hub.current
-        if hub.get_integration(DjangoIntegration) is None:
-            return real_execute(self, sql, params)
-
         with record_sql_queries(
-            hub, self.cursor, sql, params, paramstyle="format", executemany=False
+            cursor=self.cursor,
+            query=sql,
+            params_list=params,
+            paramstyle="format",
+            executemany=False,
+            span_origin=DjangoIntegration.origin_db,
         ) as span:
             _set_db_data(span, self)
-            if hub.client:
-                options = hub.client.options["_experiments"].get("attach_explain_plans")
-                if options is not None:
-                    attach_explain_plan_to_span(
-                        span,
-                        self.cursor.connection,
-                        sql,
-                        params,
-                        self.mogrify,
-                        options,
-                    )
             result = real_execute(self, sql, params)
 
         with capture_internal_exceptions():
-            add_query_source(hub, span)
+            add_query_source(span)
 
         return result
 
+    @ensure_integration_enabled(DjangoIntegration, real_executemany)
     def executemany(self, sql, param_list):
         # type: (CursorWrapper, Any, List[Any]) -> Any
-        hub = Hub.current
-        if hub.get_integration(DjangoIntegration) is None:
-            return real_executemany(self, sql, param_list)
-
         with record_sql_queries(
-            hub, self.cursor, sql, param_list, paramstyle="format", executemany=True
+            cursor=self.cursor,
+            query=sql,
+            params_list=param_list,
+            paramstyle="format",
+            executemany=True,
+            span_origin=DjangoIntegration.origin_db,
         ) as span:
             _set_db_data(span, self)
 
             result = real_executemany(self, sql, param_list)
 
         with capture_internal_exceptions():
-            add_query_source(hub, span)
+            add_query_source(span)
 
         return result
 
+    @ensure_integration_enabled(DjangoIntegration, real_connect)
     def connect(self):
         # type: (BaseDatabaseWrapper) -> None
-        hub = Hub.current
-        if hub.get_integration(DjangoIntegration) is None:
-            return real_connect(self)
-
         with capture_internal_exceptions():
-            hub.add_breadcrumb(message="connect", category="query")
+            sentry_sdk.add_breadcrumb(message="connect", category="query")
 
-        with hub.start_span(op=OP.DB, description="connect") as span:
+        with sentry_sdk.start_span(
+            op=OP.DB,
+            name="connect",
+            origin=DjangoIntegration.origin_db,
+        ) as span:
             _set_db_data(span, self)
             return real_connect(self)
 
@@ -687,7 +698,6 @@ def install_sql_hook():
 
 def _set_db_data(span, cursor_or_db):
     # type: (Span, Any) -> None
-
     db = cursor_or_db.db if hasattr(cursor_or_db, "db") else cursor_or_db
     vendor = db.vendor
     span.set_data(SPANDATA.DB_SYSTEM, vendor)
@@ -705,15 +715,20 @@ def _set_db_data(span, cursor_or_db):
     if is_psycopg2:
         connection_params = cursor_or_db.connection.get_dsn_parameters()
     else:
-        is_psycopg3 = (
-            hasattr(cursor_or_db, "connection")
-            and hasattr(cursor_or_db.connection, "info")
-            and hasattr(cursor_or_db.connection.info, "get_parameters")
-            and inspect.isroutine(cursor_or_db.connection.info.get_parameters)
-        )
-        if is_psycopg3:
-            connection_params = cursor_or_db.connection.info.get_parameters()
-        else:
+        try:
+            # psycopg3, only extract needed params as get_parameters
+            # can be slow because of the additional logic to filter out default
+            # values
+            connection_params = {
+                "dbname": cursor_or_db.connection.info.dbname,
+                "port": cursor_or_db.connection.info.port,
+            }
+            # PGhost returns host or base dir of UNIX socket as an absolute path
+            # starting with /, use it only when it contains host
+            pg_host = cursor_or_db.connection.info.host
+            if pg_host and not pg_host.startswith("/"):
+                connection_params["host"] = pg_host
+        except Exception:
             connection_params = db.get_connection_params()
 
     db_name = connection_params.get("dbname") or connection_params.get("database")
@@ -726,8 +741,18 @@ def _set_db_data(span, cursor_or_db):
 
     server_port = connection_params.get("port")
     if server_port is not None:
-        span.set_data(SPANDATA.SERVER_PORT, text_type(server_port))
+        span.set_data(SPANDATA.SERVER_PORT, str(server_port))
 
     server_socket_address = connection_params.get("unix_socket")
     if server_socket_address is not None:
         span.set_data(SPANDATA.SERVER_SOCKET_ADDRESS, server_socket_address)
+
+
+def add_template_context_repr_sequence():
+    # type: () -> None
+    try:
+        from django.template.context import BaseContext
+
+        add_repr_sequence_type(BaseContext)
+    except Exception:
+        pass
